package jartege ; 

/**
 * Cette classe permet de generer l'ensemble des n-uplets de la forme
 * (n_1, n_2, ... n_p) tel que : 
 *    . pour tout i, n_i est un entier positif ou nul ; 
 *    . n_1 + n_2 + ... + n_p = v.
 */

import java.util.Iterator ; 

class IndexEnumerator {

   /**
    * Invariant.
    * <pre><jml>
    *    invariant getP() >= 1 && getV() >= 0 ; 
    * </jml></pre>
    */

   // Taille des n-uplets
   final private int p ; 

   // Valeur de la somme des n-uplets
   final private int v ; 

   // Le n-uplet courant : prochain n-uplet renvoye par next()
   private int[] cour ; 
 
   // Indique la prochaine valeur de hasNext() 
   private boolean fini ; 

   // Reste de la somme a repartir
   private int[] rem ; 
   

   /**
    * Constructeur d'enumerateur de n-uplet.
    * <pre><jml>
    *    requires p >= 1 && v >= 0 ; 
    * </jml></pre>
    */

   IndexEnumerator(int p, int v) {
      this.p = p ; 
      this.v = v ; 
      cour = new int[p] ; 
      rem = new int[p] ; 
      for (int i = 0 ; i < p - 1 ; i++) {
         cour[i] = 0 ; 
         rem[i] = v ; 
      }
      cour[p - 1] = v ; 
      rem[p - 1] = 0 ; 
      fini = false ; 
      
   }
      
   /**
    * Retourne la valeur p.
    */
   /*@ pure */
   public int getP() {
      return p ; 
   }

   /**
    * Retourne la valeur v.
    */
   /*@ pure */
   public int getV() {
      return v ; 
   }

   /**
    * Retourne vrai si et seulement si l'numeration a encore des n-uplets. 
    */
   /*@ pure */
   public boolean hasNext() {
      return !fini ; 
   }

   /**
    * Retourne le n-uplet suivant de l'numration.
    * <pre><jml>
    *    requires hasNext() ; 
    *    ensures \result != null && \result.length == getP() ; 
    *    ensures (\sum int i ; 0 <= i && i < getP() ; \result[i]) == getV() ; 
    * </jml></pre>
    */
   public int[] next() {
      // Le resultat
      int[] result ; 

      if (fini) {
         // Cas o on ne devrait pas appelet next()
         result = null ; 
      } else {
         // Cas normal
         // On construit le rsultat
         result = new int[p] ; 
         for (int i = 0 ; i < p ; i++) {
            result[i] = cour[i] ; 
         }

         // On calcule le prochain n-uplet

         int niv = p - 1 ; // Niveau courant

         while (niv > 0 && rem[niv] == 0) {
            niv-- ; 
         }

         if (niv == 0 && cour[0] == v) {
            fini = true ; 
         } else {
            cour[niv]++ ; 
            rem[niv]-- ; 
            niv++ ; 
            while (niv < p - 1) {
               cour[niv] = 0 ; 
               rem[niv] = rem[niv - 1] ; 
               niv++ ; 
            }
            cour[p - 1] = rem[p - 2] ; 
         }
      }
      return result ; 
   }

   public void remove() { } 


}

